/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2020  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <assert.h>

#include "conversion.h"

QByteArray Conversion::Ascii(
	const unsigned int 					index,
	const QByteArray					&array)
{
	QByteArray							data;
	unsigned int						cntr;
	
	assert(index < static_cast<unsigned int>(array.size()));
	
	for(cntr = index;cntr < static_cast<unsigned int>(array.size());cntr++)
	{
		data.push_back(array[cntr]);
		
		if(array[cntr] == 0x00)
		{
			return data;
		}
	}
	
	return data;
}

double Conversion::Double(
	const unsigned int 					index,
	const QByteArray					&array)
{
	double								d;
	char								*c;
	unsigned int						cntr;
		
	assert((index + sizeof(double) -1) < static_cast<unsigned int>(array.size()));
	
	c = (char*)(&d);
	
	for(cntr = 0;cntr < sizeof(double);cntr++)
	{
		*(c + cntr) = array[index + cntr];
	}
	
	return d;
}

float Conversion::Float(
	const unsigned int 					index,
	const QByteArray					&array)
{
	float								f;
	char								*c;
	unsigned int						cntr;
	
	assert((index + sizeof(float) -1) < static_cast<unsigned int>(array.size()));
	
	c = (char*)(&f);
	
	for(cntr = 0;cntr < sizeof(float);cntr++)
	{
		*(c + cntr) = array[index + cntr];
	}
	
	return f;
}


short Conversion::Short_Integer(
	const unsigned int 					index,
	const QByteArray					&array)
{
	short								s;
	char								*c;
	unsigned int						cntr;
	
	assert((index + sizeof(short) -1) < static_cast<unsigned int>(array.size()));
	
	c = (char*)(&s);
	
	for(cntr = 0;cntr < sizeof(short);cntr++)
	{
		*(c + cntr) = array[index + cntr];
	}
	
	return s;
}


int Conversion::Integer(
	const unsigned int 					index,
	const QByteArray					&array)
{
	int									i;
	char								*c;
	unsigned int						cntr;
	
	assert((index + sizeof(int) -1) < static_cast<unsigned int>(array.size()));
	
	c = (char*)(&i);
	
	for(cntr = 0;cntr < sizeof(int);cntr++)
	{
		*(c + cntr) = array[index + cntr];
	}
	
	return i;
}

qint64 Conversion::Long_Integer(
	const unsigned int 					index,
	const QByteArray					&array)
{
	qint64								i;
	char								*c;
	unsigned int						cntr;
	
	assert((index + sizeof(qint64) -1) < static_cast<unsigned int>(array.size()));
	
	c = (char*)(&i);
	
	for(cntr = 0;cntr < sizeof(qint64);cntr++)
	{
		*(c + cntr) = array[index + cntr];
	}
	
	return i;
}

void Conversion::Set_Ascii(
	const unsigned int					index,
	const QByteArray					&data,
	QByteArray							*array)
{
	unsigned int						cntr;
	
	assert(array);
	assert((index + data.size() -1) < static_cast<unsigned int>(array->size()));

	for(cntr = 0;cntr < static_cast<unsigned int>(data.size());cntr++)
	{
		array->operator[](index + cntr) = data[cntr];
	}
}

void Conversion::Set_Double(
	const unsigned int					index,
	const double 						&data,
	QByteArray							*array)
{
	unsigned int						cntr;
	char								*c;
	double								d(data);
	
	assert(array);
	assert((index + sizeof(double) -1) < static_cast<unsigned int>(array->size()));
	
	c = (char*)(&d);
	
	for(cntr = 0;cntr < sizeof(double);cntr++)
	{
		array->operator[](index + cntr) = *(c + cntr);
	}
	
}

void Conversion::Set_Float(
	const unsigned int 					index,
	const float 						&data,
	QByteArray							*array)
{
	unsigned int						cntr;
	char								*c;
	float								f(data);
	
	assert(array);
	assert((index + sizeof(float) -1) < static_cast<unsigned int>(array->size()));
	
	c = (char*)(&f);
	
	for(cntr = 0;cntr < sizeof(float);cntr++)
	{
		array->operator[](index + cntr) = *(c + cntr);
	}
}

void Conversion::Set_Short_Integer(
	const unsigned int 					index,
	const short 						&data,
	QByteArray							*array)
{
	unsigned int						cntr;
	char								*c;
	short								s(data);
	
	assert(array);
	assert((index + sizeof(short) -1) < static_cast<unsigned int>(array->size()));
	
	c = (char*)(&s);
	
	for(cntr = 0;cntr < sizeof(short);cntr++)
	{
		array->operator[](index + cntr) = *(c + cntr);
	}
}

void Conversion::Set_Integer(
	const unsigned int 					index,
	const int							&data,
	QByteArray							*array)
{
	unsigned int						cntr;
	char								*c;
	int									i(data);
	
	assert(array);
	assert((index + sizeof(int) -1) < static_cast<unsigned int>(array->size()));
	
	c = (char*)(&i);
	
	for(cntr = 0;cntr < sizeof(int);cntr++)
	{
		array->operator[](index + cntr) = *(c + cntr);
	}
}

void Conversion::Set_Long_Integer(
	const unsigned int 					index,
	const qint64						&data,
	QByteArray							*array)
{
	unsigned int						cntr;
	char								*c;
	qint64								i(data);
	
	assert(array);
	assert((index + sizeof(qint64) -1) < static_cast<unsigned int>(array->size()));
	
	c = (char*)(&i);
	
	for(cntr = 0;cntr < sizeof(qint64);cntr++)
	{
		array->operator[](index + cntr) = *(c + cntr);
	}
}

