/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2020  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <assert.h>

#include "data.h"

int TData::d_next_page_id				= 0;

TData::TData(void)
{
	d_active_page_index = -1;
	d_next_page_id = 1;
}

TData::~TData(void)
{
}

int	TData::Active_Page(void) const
{
	if(d_active_page_index < 0)
	{
		return -1;
	}
	
	if(!(static_cast<unsigned int>(d_active_page_index) < d_page_items.size()))
	{
		return -1;
	}
	
	return static_cast<int>(d_page_items[d_active_page_index].id);
}

int TData::Page_Id(
	const int							index) const
{
	assert(static_cast<unsigned int>(index) < d_page_items.size());

	return d_page_items[index].id;
}

int TData::Item_Count(void) const
{
	assert(!(d_active_page_index < 0));
	assert(static_cast<unsigned int>(d_active_page_index) < d_page_items.size());

	return static_cast<int>(d_page_items[d_active_page_index].data_items.size());
}

int TData::Item_Max_Value(void) const
{
	int									max_value(0);
	
	std::vector<TDataItem>::const_iterator iter;
	
	assert(!(d_active_page_index < 0));
	assert(static_cast<unsigned int>(d_active_page_index) < d_page_items.size());
	
	for(iter = d_page_items[d_active_page_index].data_items.begin();iter != d_page_items[d_active_page_index].data_items.end();++iter)
	{
		if((*iter).value > max_value)
		{
			max_value = (*iter).value;
		}
	}
	
	return max_value;
}

bool TData::Item_Exists(
	const int							id) const
{
	std::vector<TDataItem>::const_iterator iter;
	
	assert(!(d_active_page_index < 0));
	assert(static_cast<unsigned int>(d_active_page_index) < d_page_items.size());
	
	for(iter = d_page_items[d_active_page_index].data_items.begin();iter != d_page_items[d_active_page_index].data_items.end();++iter)
	{
		if(id == (*iter).id)
		{
			return true;
		}
	}
	
	return false;
}

TDataItem TData::Item(
	const int							index) const
{
	assert(!(d_active_page_index < 0));
	assert(static_cast<unsigned int>(d_active_page_index) < d_page_items.size());
	assert(static_cast<unsigned int>(index) < d_page_items[d_active_page_index].data_items.size());
	
	return d_page_items[d_active_page_index].data_items[index];
}

void TData::Clear(void)
{
	d_page_items.clear();
	
	d_active_page_index = -1;
	d_next_page_id = 1;
}

QImage TData::Page_Image(void) const
{
	assert(!(d_active_page_index < 0));
	assert(static_cast<unsigned int>(d_active_page_index) < d_page_items.size());
	
	return d_page_items[d_active_page_index].image;
}

bool TData::Page_Image_Valid(void) const
{
	if(d_active_page_index < 0)
	{
		return false;
	}
	
	if(!(static_cast<unsigned int>(d_active_page_index) < d_page_items.size()))
	{
		return false;
	}
	
	return (!d_page_items[d_active_page_index].image.isNull());
}

bool TData::Page_Has_Data(void) const
{
	if(d_active_page_index < 0)
	{
		return false;
	}
	
	if(!(static_cast<unsigned int>(d_active_page_index) < d_page_items.size()))
	{
		return false;
	}
	
	return (d_page_items[d_active_page_index].data_items.size() > 0);
}

int TData::Add_Page(void)
{
	TPageItem							page_item;
	
	page_item.id = d_next_page_id++;
	d_page_items.push_back(page_item);
	
	return page_item.id;
}

int TData::Add_Page(
	const QImage						&image)
{
	TPageItem							page_item;
	
	page_item.image = image;
	page_item.id = d_next_page_id++;
	d_page_items.push_back(page_item);
	
	return page_item.id;
}

void TData::Set_Page_Image(
	const QImage						&image)
{
	assert(!(d_active_page_index < 0));
	assert(static_cast<unsigned int>(d_active_page_index) < d_page_items.size());
	
	d_page_items[d_active_page_index].image = image;
}

void TData::Remove_Page(
	const int							page_id)
{
	int									index;
	
	index = this->Page_Index(page_id);
	
	if(index < 0)
	{
		return;
	}
	
	d_page_items.erase(d_page_items.begin() + index);
	
	if(d_active_page_index == index)
	{
		d_active_page_index = -1;
	}
	else if(d_active_page_index > index)
	{
		--d_active_page_index;
	}
}

bool TData::Set_Active_Page(
	const int							page_id)
{
	d_active_page_index = this->Page_Index(page_id);
	
	return (!(d_active_page_index < 0));
}

bool TData::Add_Item(
	const TDataItem						&item)
{	
	assert(!(d_active_page_index < 0));
	assert(static_cast<unsigned int>(d_active_page_index) < d_page_items.size());

	if(this->Item_Exists(item.id))
	{
		return false;
	}
	
	d_page_items[d_active_page_index].data_items.push_back(item);
		
	return true;
}

bool TData::Remove_Item(
	const int							id)
{
	std::vector<TDataItem>::iterator iter;

	assert(!(d_active_page_index < 0));
	assert(static_cast<unsigned int>(d_active_page_index) < d_page_items.size());

	for(iter = d_page_items[d_active_page_index].data_items.begin();iter != d_page_items[d_active_page_index].data_items.end();++iter)
	{
		if(id == (*iter).id)
		{
			d_page_items[d_active_page_index].data_items.erase(iter);
			return true;
		}
	}
	
	return false;
}

void TData::Clear_Selection(void)
{
	std::vector<TDataItem>::iterator iter;
	
	assert(!(d_active_page_index < 0));
	assert(static_cast<unsigned int>(d_active_page_index) < d_page_items.size());
	
	for(iter = d_page_items[d_active_page_index].data_items.begin();iter != d_page_items[d_active_page_index].data_items.end();++iter)
	{
		(*iter).is_selected = false;
	}

}

bool TData::Change_Item_Value(
	const int							id,
	const int							new_value)
{
	std::vector<TDataItem>::iterator iter;
	
	assert(!(d_active_page_index < 0));
	assert(static_cast<unsigned int>(d_active_page_index) < d_page_items.size());

	for(iter = d_page_items[d_active_page_index].data_items.begin();iter != d_page_items[d_active_page_index].data_items.end();++iter)
	{
		if(id == (*iter).id)
		{
			(*iter).value = new_value;
			return true;
		}
	}
	
	return false;
}

bool TData::Change_Item_Text(
	const int							id,
	const QString						&new_text)
{
	std::vector<TDataItem>::iterator iter;
	
	assert(!(d_active_page_index < 0));
	assert(static_cast<unsigned int>(d_active_page_index) < d_page_items.size());

	for(iter = d_page_items[d_active_page_index].data_items.begin();iter != d_page_items[d_active_page_index].data_items.end();++iter)
	{
		if(id == (*iter).id)
		{
			(*iter).text = new_text;
			return true;
		}
	}
	
	return false;
}

bool TData::Change_Item_Border(
	const int							id,
	const int							new_border_type)
{
	std::vector<TDataItem>::iterator iter;
	
	assert(!(d_active_page_index < 0));
	assert(static_cast<unsigned int>(d_active_page_index) < d_page_items.size());

	for(iter = d_page_items[d_active_page_index].data_items.begin();iter != d_page_items[d_active_page_index].data_items.end();++iter)
	{
		if(id == (*iter).id)
		{
			(*iter).border_type = new_border_type;
			return true;
		}
	}
	
	return false;
}

bool TData::Change_Item_Position(
	const int							id,
	const QPointF						&new_position)
{
	std::vector<TDataItem>::iterator iter;
	
	assert(!(d_active_page_index < 0));
	assert(static_cast<unsigned int>(d_active_page_index) < d_page_items.size());
	
	for(iter = d_page_items[d_active_page_index].data_items.begin();iter != d_page_items[d_active_page_index].data_items.end();++iter)
	{
		if(id == (*iter).id)
		{
			(*iter).position = new_position;
			return true;
		}
	}
	
	return false;
}

bool TData::Change_Item_Selection(
	const int							id,
	const bool							state)
{
	std::vector<TDataItem>::iterator iter;
	
	assert(!(d_active_page_index < 0));
	assert(static_cast<unsigned int>(d_active_page_index) < d_page_items.size());
	
	for(iter = d_page_items[d_active_page_index].data_items.begin();iter != d_page_items[d_active_page_index].data_items.end();++iter)
	{
		if(id == (*iter).id)
		{
			(*iter).is_selected = state;
			return true;
		}
	}
	
	return false;
}

int TData::Page_Index(
	const int							page_id) const
{
	int									index(-1);
	
	std::vector<TPageItem>::const_iterator iter;
	
	for(iter = d_page_items.begin();iter != d_page_items.end();++iter)
	{
		++index;
		
		if((*iter).id == page_id)
		{
			return index;
		}
	}
	
	return -1;
}
