/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2020  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef DATAHEADERFILE
#define DATAHEADERFILE

#include <QString>
#include <QPointF>
#include <QImage>
#include <vector>

#include "datatypes.h"

class TData
{
public:
	
// CREATORS
	TData(void);
	~TData(void);
	
// ACCESSORS
	int	Active_Page(void) const;		// returns page_id
	int Page_Count(void) const {return static_cast<int>(d_page_items.size());}
	int Page_Id(const int index) const;	// returns page_id
	
	QImage Page_Image(void) const;
	bool Page_Image_Valid(void) const;
	bool Page_Has_Data(void) const;
														
	int Item_Count(void) const;
	int Item_Max_Value(void) const;
	bool Item_Exists(const int id) const;
	
	TDataItem Item(const int index) const;	// uses index not id
	
// MANIPULATORS
	void Clear(void);
	int Add_Page(void);					// returns page_id
	int Add_Page(const QImage &image);	// returns page_id
	void Set_Page_Image(const QImage &image);
	void Remove_Page(const int page_id);
	bool Set_Active_Page(const int page_id);
														
	bool Add_Item(const TDataItem &item);
	bool Remove_Item(const int id);
	void Clear_Selection(void);
	bool Change_Item_Value(const int id, const int new_value);
	bool Change_Item_Text(const int id, const QString &new_text);
	bool Change_Item_Border(const int id, const int new_border_type);
	bool Change_Item_Position(const int id, const QPointF &new_position);
	bool Change_Item_Selection(const int id, const bool state);
	
private:
	std::vector<TPageItem>				d_page_items;
	int									d_active_page_index;
	
	static int							d_next_page_id;
	
	int Page_Index(const int page_id) const;
	
// NOT IMPLEMENTED
	
};

#endif