/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2020  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef DISPLAYWIDGETHEADERFILE
#define DISPLAYWIDGETHEADERFILE

#include <QGraphicsView>
#include <QPixmap>
#include <QSize>
#include <QString>
#include <QPointF>
#include <vector>

class QGraphicsScene;
class QGraphicsPixmapItem;
class QMouseEvent;

class TDisplayWidget : public QGraphicsView
{
Q_OBJECT
public:
	
// CREATORS
	TDisplayWidget(const QWidget *parent=0);
	virtual ~TDisplayWidget(void);
	
// ACCESSORS
	bool Item_Exists(const int id) const;
	QPointF Item_Position(const int id) const;
	
	QPixmap Pixmap(void);
	QPixmap Background_Pixmap(void) const;
	QSize Label_Size(void) const;

// MANIPULATORS
	void Change_Scale(const double &scale);
	void Set_Label_Size(const QSize &size,const int font_size);
	void Set_Label_Color(const QColor &color);
	void Set_Track_Label(const bool state) {d_track_label = state;}

	void Set_Selected_Items(const std::vector<int> &items,const bool allow_center_on_select);
	
	void Clear(void);
	void Remove_Items(void);
	void Remove_Item(const int id);
	
	void Set_Background_Image(const QPixmap &pixmap);
	
	QGraphicsPixmapItem* Add_Item(const int id,const int value,const QString &text,const int border_type);
	void Move_Item(const int id,const QPointF &pos);

	void Change_Item_Border(const int id,const int border_type);
	void Change_Item_Value(const int id, const int value);
	void Change_Item_Text(const int id, const QString &text);
	
signals:
	void Right_Mouse_Click(const QPointF&);
	void Item_Clicked(const int);
	void Update_Item_Position(const int,const QPointF&);
	
protected:
	
	virtual void mouseDoubleClickEvent(QMouseEvent *event);
	virtual void mouseMoveEvent(QMouseEvent *event);
	virtual void mousePressEvent(QMouseEvent *event);
	virtual void mouseReleaseEvent(QMouseEvent *event);

private:
	QGraphicsScene						*d_graphics_scene;
	QGraphicsPixmapItem					*d_background_image;
	QGraphicsPixmapItem					*d_moving_item;
	
	QSize								d_label_item_size;
	int									d_font_size;
	mutable QPixmap						d_item_pixmap;
	std::vector<QGraphicsPixmapItem*>	d_label_item_pixmaps;
	bool								d_moving_item_moved;
	Qt::MouseButtons					d_mouse_down_buttons;
	QPointF								d_moving_item_last_position;
	QColor								d_label_color;
	bool								d_track_label;
	
	void Update_Scene_Rect(void);

	QPixmap Render_Pixmap(const int value,const int &border_type,const bool is_selected) const;
	QPixmap Render_Pixmap(const QString &text,const int &border_type,const bool is_selected) const;
	
// NOT IMPLEMENTED
	TDisplayWidget(const TDisplayWidget&);
	TDisplayWidget& operator=(const TDisplayWidget&);
};

#endif

