/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2020  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#ifndef LABELWIDGETHEADERFILE
#define LABELWIDGETHEADERFILE

#include <QWidget>
#include <vector>

#include "data.h"

class QComboBox;
class QLineEdit;
class QListWidget;
class QSlider;
class QToolButton;

class TLabelWidget : public QWidget
{
Q_OBJECT

public:
	
// CREATORS
	TLabelWidget(const QWidget *parent=0, Qt::WindowFlags flags=0);
	~TLabelWidget(void);

// ACCESSORS
	int Border_Type(void) const;
	int Next_Label_Index(void) const {return d_current_value++;}
	std::vector<int> Selected_Items(void) const;	// list of selected items

// MANIPULATORS
	void Set_Border_Type(const int border_type);
	void Set_Label_Index(const int label_index);
	void Set_Display_Format(const QString &format) {d_display_format = format;}
	
	void Clear(void);
	void Add_Item(const TDataItem &data_item);
	void Remove_Item(const int id);
	
	void Clear_Selection(void);
	void Set_Selection(const int id);
	void Add_Selection(const int id);

signals:
	void Add_Item(void);
	void Remove_Item(void);
	void Change_Item_Value(const int,const int);		// id, new value
	void Change_Item_Text(const int,const QString&);	// id, new text
	void Change_Item_Border(const int,const int);		// id, TBorderType
	void Selection_Changed(void);

private slots:
	void Item_Selection_Changed(void);
	void Border_Type_Changed(void);
	void Current_Value_Changed(const QString&);
	void Current_Text_Changed(void);

private:
	QComboBox							*d_item_border_combo;
	QLineEdit							*d_item_value_edit;
	QLineEdit							*d_item_text_edit;
	QListWidget							*d_item_list;
	QToolButton							*d_add_item_button;
	QToolButton							*d_remove_item_button;
	
	QString								d_display_format;
	mutable int							d_current_value;
	
// NOT IMPLEMENTED
	TLabelWidget(const TLabelWidget&);
	TLabelWidget& operator=(const TLabelWidget&);
};

#endif
