/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2020  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#ifndef PAGEWIDGETHEADERFILE
#define PAGEWIDGETHEADERFILE

#include <QWidget>
#include <QList>

class QListWidget;
class QToolButton;
class QLineEdit;

class TPageWidget : public QWidget
{
Q_OBJECT

public:

// CREATORS
	TPageWidget(const QWidget *parent=0, Qt::WindowFlags flags=0);
	~TPageWidget(void);

// ACCESSORS
	QList<int> Page_Ids(void) const;		// page_id's in their current order
	QString Page_Name(const int page_id) const;

// MANIPULATORS
	void Clear(void);
	void Add_Page(const int page_id);
	void Remove_Page(const int page_id);
	void Set_Active_Page(const int page_id);
	void Set_Page_Name(const int page_id, const QString &name);

signals:
	void Add_Page(void);
	void Remove_Pages(const QList<int>&);	// page_ids
	void Selection_Changed(int);			// page_id

private slots:
	void Page_Selection_Changed(void);
	void Remove_Page(void);
	void Page_Name_Edited(const QString&);

private:
	QListWidget							*d_page_list;
	QToolButton							*d_add_page_button;
	QToolButton							*d_remove_page_button;
	QLineEdit							*d_page_name_edit;

// NOT IMPLEMENTED
	TPageWidget(const TPageWidget&);
	TPageWidget& operator=(const TPageWidget&);
};

#endif
