/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2020  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#include <QApplication>
#include <QComboBox>
#include <QFrame>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QRadioButton>
#include <QSpacerItem>
#include <QStringList>
#include <QVariant>
#include <vector>

#include "datatypes.h"

#include "reindexdialog.h"

TReindexDialog::TReindexDialog(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QDialog(const_cast<QWidget*>(parent),flags)
{
	QFrame								*separator_hline_bottom;
	QFrame								*separator_hline_mid;
	QFrame								*separator_hline_top;
	QFrame								*separator_vline_options;
	QFrame								*separator_vline_parameters;
	QGridLayout							*dialog_layout;
	QHBoxLayout							*button_hlayout;
	QLabel								*first_item_value_label;
	QLabel								*increment_over_page_label;
	QLabel								*options_label;
	QLabel								*parameters_label;
	QPushButton							*cancel_button;
	QSpacerItem							*button_hspacer;
	QSpacerItem							*control_hspacer;
	QSpacerItem							*dialog_vspacer;


	this->resize(520,250);

	dialog_layout = new QGridLayout(this);

	separator_hline_top = new QFrame(this);
	separator_hline_top->setFrameShape(QFrame::HLine);
	separator_hline_top->setFrameShadow(QFrame::Sunken);
	dialog_layout->addWidget(separator_hline_top,0,0,1,5);

	parameters_label = new QLabel(this);
	parameters_label->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
	dialog_layout->addWidget(parameters_label,1,0,2,1);

	separator_vline_parameters = new QFrame(this);
	separator_vline_parameters->setFrameShape(QFrame::VLine);
	separator_vline_parameters->setFrameShadow(QFrame::Sunken);
	dialog_layout->addWidget(separator_vline_parameters,1,1,2,1);

	first_item_value_label = new QLabel(this);
	dialog_layout->addWidget(first_item_value_label,1,2,1,1);

	d_first_item_value_edit = new QLineEdit(this);
	d_first_item_value_edit->setAlignment(Qt::AlignCenter);
	dialog_layout->addWidget(d_first_item_value_edit,1,3,1,1);

	control_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	dialog_layout->addItem(control_hspacer,1,4,2,1);

	increment_over_page_label = new QLabel(this);
	dialog_layout->addWidget(increment_over_page_label,2,2,1,1);

	d_page_increment_method_combo = new QComboBox(this);
	dialog_layout->addWidget(d_page_increment_method_combo,2,3,1,1);

	separator_hline_mid = new QFrame(this);
	separator_hline_mid->setFrameShape(QFrame::HLine);
	separator_hline_mid->setFrameShadow(QFrame::Sunken);
	dialog_layout->addWidget(separator_hline_mid,3,0,1,5);

	options_label = new QLabel(this);
	options_label->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
	dialog_layout->addWidget(options_label,4,0,3,1);

	separator_vline_options = new QFrame(this);
	separator_vline_options->setFrameShape(QFrame::VLine);
	separator_vline_options->setFrameShadow(QFrame::Sunken);
	dialog_layout->addWidget(separator_vline_options,4,1,3,1);

	d_all_items_check = new QRadioButton(this);
	d_all_items_check->setChecked(true);
	dialog_layout->addWidget(d_all_items_check,4,2,1,3);

	d_current_page_check = new QRadioButton(this);
	dialog_layout->addWidget(d_current_page_check,5,2,1,3);

	d_current_selected_check = new QRadioButton(this);
	dialog_layout->addWidget(d_current_selected_check,6,2,1,3);

	separator_hline_bottom = new QFrame(this);
	separator_hline_bottom->setFrameShape(QFrame::HLine);
	separator_hline_bottom->setFrameShadow(QFrame::Sunken);
	dialog_layout->addWidget(separator_hline_bottom,7,0,1,5);

	dialog_vspacer = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);
	dialog_layout->addItem(dialog_vspacer,8,0,1,5);

	button_hlayout = new QHBoxLayout();

	button_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	button_hlayout->addItem(button_hspacer);

	cancel_button = new QPushButton(this);
	cancel_button->setAutoDefault(false);
	button_hlayout->addWidget(cancel_button);

	d_apply_button = new QPushButton(this);
	d_apply_button->setAutoDefault(false);
	button_hlayout->addWidget(d_apply_button);
	dialog_layout->addLayout(button_hlayout,9,0,1,5);

	this->setWindowTitle(QStringLiteral("Reindex Items"));
	
	parameters_label->setText(QStringLiteral("Parameters:"));
	first_item_value_label->setText(QStringLiteral("First Item Value:"));
	increment_over_page_label->setText(QStringLiteral("Increment Over Page:"));
	options_label->setText(QStringLiteral("Options:"));
	d_all_items_check->setText(QStringLiteral("All Pages, All Label Items"));
	d_current_page_check->setText(QStringLiteral("Current Page, All Label Items"));
	d_current_selected_check->setText(QStringLiteral("Current Page, Selected Label Items"));
	cancel_button->setText(QStringLiteral("Cancel"));
	d_apply_button->setText(QStringLiteral("Apply"));
	
	d_page_increment_method_combo->addItem(QStringLiteral("Next 1X"),TIncrementOverPageMethod::INCREMENT_1X);
	d_page_increment_method_combo->addItem(QStringLiteral("Next 10X"),TIncrementOverPageMethod::INCREMENT_10X);
	d_page_increment_method_combo->addItem(QStringLiteral("Next 100X"),TIncrementOverPageMethod::INCREMENT_100X);
	d_page_increment_method_combo->addItem(QStringLiteral("Next 1000X"),TIncrementOverPageMethod::INCREMENT_1000X);

	// defaults
	d_page_increment_method_combo->setCurrentIndex(2);	// 100x
	d_first_item_value_edit->setText(QStringLiteral("100"));
	
	connect(cancel_button,SIGNAL(clicked(void)),this,SLOT(reject(void)));
	connect(d_apply_button,SIGNAL(clicked(void)),this,SLOT(accept(void)));
}

TReindexDialog::~TReindexDialog(void)
{
}

int TReindexDialog::Reindex_Option(void) const
{
	int									option(TReindexOption::OPTION_ALL_ITEMS);
	
	if(d_current_page_check->isChecked())
	{
		option = TReindexOption::OPTION_PAGE_ITEMS;
	}
	else if(d_current_selected_check->isChecked())
	{
		option = TReindexOption::OPTION_SELECTED_ITEMS;
	}
	
	return option;
}

int TReindexDialog::Increment_Over_Page_Method(void) const
{
	return d_page_increment_method_combo->currentData(Qt::UserRole).toInt();
}

int TReindexDialog::Initial_Value(void) const
{
	int									ival;
	bool								state;
	
	ival = d_first_item_value_edit->text().toInt(&state);
	
	if(!state)
	{
		ival = 1;	// default?
	}
	
	return ival;
}


