/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2020  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#ifndef REPORTROADMAPHEADERFILE
#define REPORTROADMAPHEADERFILE

#include <QMainWindow>
#include <QPointF>
#include <QDir>
#include <QList>
#include <vector>

#include "data.h"

class QAction;
class QDockWidget;
class QLabel;
class QSlider;
class QCloseEvent;
class QSettings;
class TLabelWidget;
class TSettingsWidget;
class TDisplayWidget;
class TPageWidget;
class TReindexDialog;
class TReportHeaderDialog;
class TMessageBox;
class TXmlFile;

class TReportRoadMap : public QMainWindow
{
Q_OBJECT

public:

// CREATORS
	TReportRoadMap(const QWidget *parent=0, Qt::WindowFlags flags=0);
	virtual ~TReportRoadMap(void);

// ACCESSORS

// MANIPULATORS

signals:

private slots:
	
	void File_New(void);
	void File_Open(void);
	void File_Import(void);
	void File_Save(void);
	void File_SaveAs(void);
	void File_Export_Images(void);
	void File_Export_Pdf(void);
	void Edit_Reindex(void);
	void Help_About(void);

	void Image_Zoom_Changed(int);
	
	void Label_Size_Changed(int);
	void Color_Changed(const QColor);
	void Item_Display_Format_Changed(const QString&);
	void Track_Item_Option_Changed(bool);	// state

	void Add_Item(void);
	void Add_Item(const QPointF&);
	void Remove_Item(void);
	void Change_Item_Value(const int,const int);		// id, new value
	void Change_Item_Text(const int,const QString&);	// id, new text
	void Change_Item_Border(const int, const int);		// id, new border
	
	void Item_Selection_Changed(void);
	void Display_Item_Clicked(const int);
	void Update_Item_Position(const int,const QPointF&);

	void Add_Page(void);
	void Remove_Pages(const QList<int>&);				// list of page_ids
	void Page_Selection_Changed(int);					// page_id
	
protected:
	void closeEvent(QCloseEvent *event);

private:
	
struct TReportSettings
{
	int									orientation;	// 0 = portrait, 1 = landscape
	int									resolution;		// input range 1-4; 1 = 96 dpi, 4 = 384 dpi
	QString								report_header1;
	QString								report_header2;
};
	
	QAction								*d_action_edit;
	QAction								*d_action_export_images;
	QAction								*d_action_export_pdf;
	QAction								*d_action_import;
	QAction								*d_action_new;
	QAction								*d_action_open;
	QAction								*d_action_quit;
	QAction								*d_action_reindex;;
	QAction								*d_action_save;
	QAction								*d_action_saveas;
	QAction								*d_action_about;
	QDockWidget							*d_label_dock;
	QDockWidget							*d_settings_dock;
	QDockWidget							*d_page_dock;
	QLabel								*d_zoom_scale_label;
	QSlider 							*d_zoom_slider;

	TLabelWidget						*d_label_widget;
	TSettingsWidget						*d_settings_widget;
	TPageWidget							*d_page_widget;
	TDisplayWidget						*d_display_widget;
	TReindexDialog						*d_reindex_dialog;
	TReportHeaderDialog					*d_report_header_dialog;
	TMessageBox							*d_msg_box;
	
	QSettings							*d_settings;
	QDir								d_working_path;
	QString								d_active_file_name;
	bool								d_block_label_tracking;
	TData								d_data;
	
	mutable bool						d_data_changed;
	mutable QString						d_last_error;

	void Load_Image(const QString &file_name);
	
	bool Load_Legacy_Data(const QString &file_name);
	void Load_Legacy_V1(TXmlFile *xml_file,std::vector<TDataItem> *items);
	void Load_Legacy_V2(TXmlFile *xml_file,std::vector<TDataItem> *items);
	
	bool Save_Data(const QString &file_name);
	QByteArray Write_Items(void);
	
	bool Load_Data(const QString &file_name);
	void Read_Items(const QByteArray &data);

	bool Export_PDF(const QString &file_name,const QList<int> &page_ids,const TReportSettings &settings);
	bool Export_Images(const QString &file_path,const QList<int> &page_ids);
		
// NOT IMPLEMENTED
	TReportRoadMap(const TReportRoadMap&);
	TReportRoadMap& operator=(const TReportRoadMap&);
};

#endif
